; Automatically generated test suite,
; See libtest.sh for details.

(load-from-library "syntax-rules.scm")

(define Errors 0)

(define (check src expr result)
;  (write src) (display " ==> ") (write expr) (newline)
  (if (not (equal? expr result))
      (begin (write src)
             (display " FAILED!")
             (newline)
             (display "Expected: ")
             (write result)
             (newline)
             (display "But got:  ")
             (write expr)
             (newline)
             (set! Errors (+ 1 Errors)))))

(define-syntax test
  (syntax-rules (==>)
    ((_) #t)
    ((_ expr ==> result)
       (check 'expr expr 'result))
    ((_ expr ==> result . more)
       (begin (check 'expr expr 'result)
              (test . more)))))

(load-from-library "amk.scm")
(test
  (run* (vq) (appendo vq (_) '(a b c)))
    ==>  (() (a) (a b) (a b c))
)

(load-from-library "and-letstar.scm")
(test
  (and-let* ((a '((x . 1)))
             (a (assq 'x a)))
    (cdr a))                   ==>  1
;
  (and-let* ((a '((x . 1)))
             (a (assq 'z a)))
    (cdr a))                   ==>  #f
)

(load-from-library "bitwise-ops.scm")
(test
  (bitwise-clear   #b1010 #b1100)  ==>  #b0000
  (bitwise-or-not  #b1010 #b1100)  ==>  #b0001
  (bitwise-and-c2  #b1010 #b1100)  ==>  #b0010
  (bitwise-c2      #b1010 #b1100)  ==>  #b0011
  (bitwise-and-c1  #b1010 #b1100)  ==>  #b0100
  (bitwise-c1      #b1010 #b1100)  ==>  #b0101
  (bitwise-xor     #b1010 #b1100)  ==>  #b0110
  (bitwise-and-not #b1010 #b1100)  ==>  #b0111
  (bitwise-and     #b1010 #b1100)  ==>  #b1000
  (bitwise-xor-not #b1010 #b1100)  ==>  #b1001
  (bitwise-1       #b1010 #b1100)  ==>  #b1010
  (bitwise-or-c2   #b1010 #b1100)  ==>  #b1011
  (bitwise-2       #b1010 #b1100)  ==>  #b1100
  (bitwise-or-c1   #b1010 #b1100)  ==>  #b1101
  (bitwise-or      #b1010 #b1100)  ==>  #b1110
  (bitwise-set     #b1010 #b1100)  ==>  #b1111
  (bitwise-shift-left 1 10)        ==>  1024
  (bitwise-shift-right 10 1)       ==>  5
)

(load-from-library "combine.scm")
(test
  (combine 2 '(a b c))   ==>  ((a b) (a c) (b c))
  (combine* 2 '(a b c))  ==>  ((a a) (a b) (a c) (b b) (b c) (c c))
)

(load-from-library "cond-expand.scm")
(test
  (cond-expand (s9fes (cons 1 2)))               ==>  (1 . 2)
  (cond-expand (foo (cons 1 2)) (else (+ 1 2)))  ==>  3
)

(load-from-library "count.scm")
(test
  (count '(a (b (c)) d . e))  ==>  5
)

(load-from-library "define-record.scm")
(test
  (begin
    (define-record point (x 0) (y 0) (color #f))
    (let ((p (make-point)))
      (point-set-color! p 'yellow)
      (list (point? p)
            (point-color p))))               ==>  (#t yellow)
)

(load-from-library "depth.scm")
(test
  (depth '(a (b (c d (e)))))  ==>  4
)

(load-from-library "exists.scm")
(test
  (exists < '(9 1) '(8 2) '(7 3))  ==>  #t
  ; because (< 1 2 3)
)

(load-from-library "explode.scm")
(test
  (explode 'supernova)  ==>  (s u p e r n o v a)
)

(load-from-library "factor.scm")
(test
  (factor 24)  ==>  ((3 1) (2 3))
)

(load-from-library "factorial.scm")
(test
  (factorial 30)  ==>  265252859812191058636308480000000
)

(load-from-library "filter.scm")
(test
  (filter number? '(a 1 b 2 c 3))  ==>  (1 2 3)
)

(load-from-library "flatten.scm")
(test
  (flatten '(a (b ((c) d . e))))  ==>  (a b c d e)
)

(load-from-library "fluid-let-sr.scm")
(test
  (let ((a 0))
    (let ((f (lambda () a)))
      (fluid-let ((a 1))
        (f))))                ==>  1
)

(load-from-library "fluid-let.scm")
(test
  (let ((a 0))
    (let ((f (lambda () a)))
      (fluid-let ((a 1))
        (f))))                ==>  1
)

(load-from-library "for-all.scm")
(test
  (for-all < '(1 7) '(2 8) '(3 9))  ==>  #t
  ; because (< 1 2 3) and (< 7 8 9)
)

(load-from-library "hash-table.scm")
(test
  (let ((h (make-hash-table)))
    (hash-table-set! h "key" 'value)
    (hash-table-ref  h "key"))
                                ==>  (value)
)

(load-from-library "hyper.scm")
(test
  (hyper 4 3 3)  ==>  7625597484987
)

(load-from-library "implode.scm")
(test
  (implode '(b l a c k h o l e))  ==>  blackhole
)

(load-from-library "integer-sqrt.scm")
(test
  (integer-sqrt 10)  ==>  3
)

(load-from-library "intersection.scm")
(test
  (intersection '(v w x) '(w x y) '(x y z))  ==>  (x)
)

(load-from-library "iota.scm")
(test
  (iota 17 21)  ==>  (17 18 19 20 21)
)

(load-from-library "letrecstar.scm")
(test
  (letrec* ((a (lambda () (lambda () 1)))
            (b (a)))
    (b))                                  ==>  1
)

(load-from-library "list-to-set.scm")
(test
  (list->set '(a b c b c))  ==>  (a b c)
)

(load-from-library "make-partitions.scm")
(test
  (make-partitions 4)  ==>  ((4) (3 1) (2 2) (2 1 1) (1 1 1 1))
)

(load-from-library "matcher.scm")
(test
  (begin
    (define-matcher len
      ((())      0)
      (((_ . x)) (+ 1 (len x))))
    (len '(a b c d e f)))
                                ==>  6
  (let-matcher fac
    (((x)   (fac x 1))
     ((0 r) r)
     ((x r) (fac (- x 1) (* x r))))
    (fac 10))
                                ==>  3628800
  (begin
    (ml-match
      appnd (() x)      = x
          : ((h . t) x) = (cons h (appnd t x)))
    (appnd '(a b c) '(d e f)))
                                ==>  (a b c d e f)
)

(load-from-library "mergesort.scm")
(test
  (mergesort <= '(5 3 7 9 1))  ==>  (1 3 5 7 9)
)

(load-from-library "module.scm")
(test
  (begin ; Note: BEGIN is only needed for automatic testing
    (module math
      (define* (fact x)
        (if (= 0 x) 1 (* x (fact (- x 1))))))
    (import math (fact)
      (fact 5)))                               ==> 120
)

(load-from-library "name-to-file-name.scm")
(test
   (name->file-name "sys:stat-pipe?")   ==>  "sys_stat-pipep"
   (name->file-name "a->b")             ==>  "a-to-b"
   (name->file-name "*foo*")            ==>  "starfoostar"
)

(load-from-library "partition.scm")
(test
  (partition even? '(1 2 3 4 5))  ==>  ((2 4) (1 3 5))
)

(load-from-library "permute.scm")
(test
  (permute 2 '(a b c))   ==>  ((a b) (b a) (a c) (c a) (b c) (c b))
;
  (permute* 2 '(a b c))  ==>  ((a a) (a b) (a c) (b a) (b b) (b c)
                               (c a) (c b) (c c))
)

(load-from-library "programp.scm")
(test
  (program? '(let ((x 1)) (cons x x)))  ==>  #t
)

(load-from-library "quicksort.scm")
(test
  (quicksort <= '(5 3 7 9 1))  ==>  (1 3 5 7 9)
)

(load-from-library "read-from-string.scm")
(test
  (read-from-string "  (this \"is\" #(a) (list)) ; comment")
    ==>  ((this "is" #(a) (list)))
;
  (read-from-string "  (this \"is\" #(a) (list))  more text")
    ==>  ((this "is" #(a) (list)) . "  more text")
;
  (read-from-string ")")
    ==>  "read-from-string: unexpected closing parenthesis"
)

(load-from-library "read-line.scm")
(test
  (with-input-from-file "lib/read-line.scm" read-line)
    ==>  "; Scheme 9 from Empty Space, Function Library"
)

(load-from-library "records.scm")
(test
  (record-ref (record (list 'name "Foo") (list 'value 31415))
              'name)
    ==> "Foo"
;
  (equal? (record (list 'name "Foo") (list 'value 31415))
          (record (list 'value 31415) (list 'name "Foo")))
    ==> #t
)

(load-from-library "regex.scm")
(test
  (re-match (re-comp "^a[1-9]*z$") "a1289z")     ==>  ((0 6))
  (re-match (re-comp "a[1-9]+z") "___a123z___")  ==>  ((3 8))
  (re-match (re-comp "a[^1-9]+z") "a123z")       ==>  #f
  (re-match (re-comp "[1-9]*") "__1__")          ==>  ((2 3))
  (re-match (re-comp "[1-9]*") "_____")          ==>  ()
  (re-match (re-comp "f\\(.\\)\\(.\\)bar")
            "foobar")                      ==>  ((0 6) (1 2) (2 3))
)

(load-from-library "remove.scm")
(test
  (remp number? '(a 1 b 2 c 3))   ==>  (a b c)
  (remove '(b) '(a (b) (c) (b)))  ==>  (a (c))
  (remq 'b     '(a b c b d))      ==>  (a c d)
  (remv 3      '(3 1 2 3 1))      ==>  (1 2 1)
)

(load-from-library "replace.scm")
(test
  (replace '(x) '(y) '(lambda (x) y))  ==>  (lambda (y) y)
)

(load-from-library "sieve.scm")
(test
  (sieve 20)  ==>  (2 3 5 7 11 13 17 19)
)

(load-from-library "sort.scm")
(test
  (sort <= '(5 3 7 9 1))  ==>  (1 3 5 7 9)
)

(load-from-library "split-url.scm")
(test
  (split-url "ftp://example.org/foo.bar?a=1&b=2")
                          ==>  ("ftp"
                                "example.org"
                                "/foo.bar"
                                "bar"
                                (("a" . "1")
                                 ("b" . "2"))
                                #f)
)

(load-from-library "streams.scm")
(test
  (stream->list
    (append-streams (list->stream '(a b c))
                    (stream-iota 1 3)))
                                          ==>  (a b c 1 2 3)
  (stream->list
    (filter-stream even?
                   (stream-iota 1 10)))   ==>  (2 4 6 8 10)
  (stream->list
    (map-stream (lambda (x) (* 7 x))
                (stream-iota 1 5)))       ==>  (7 14 21 28 35)
  (stream->list
    (stream-member (lambda (x) (= 27 x))
                   (stream-iota 1 30)))   ==>  (27 28 29 30)
  (stream-extract
    5
    (stream-iota 1 10000000))             ==>  (1 2 3 4 5)
)

(load-from-library "string-case.scm")
(test
  (string-upcase "Hello, World!")    ==>  "HELLO, WORLD!"
  (string-downcase "Hello, World!")  ==>  "hello, world!"
)

(load-from-library "string-digest.scm")
(test
  (string-digest "hello")  ==>  2107915172
)

(load-from-library "string-find-last.scm")
(test
  (string-find-last "a" "aaaaa")        ==>  "a"
  (string-ci-find-last "A" "ab ac")     ==>  "ac"
  (string-find-last "ax" "ab ac")       ==>  #f
  (string-find-last-word "a" "ab a c")  ==>  "a c"
  (string-find-last-word "a" "ab ac")   ==>  #f
)

(load-from-library "string-find.scm")
(test
  (string-find "ein" "gemeinsam")     ==>  "einsam"
  (string-find "people" "democracy")  ==>  #f
  (string-find-word "me" "test me")   ==>  "me"
  (string-find-word "me" "testme")    ==>  #f
)

(load-from-library "string-last-position.scm")
(test
  (string-last-position "a" "aaaaa")        ==>  4
  (string-ci-last-position "A" "ab ac")     ==>  3
  (string-last-position "ax" "ab ac")       ==>  #f
  (string-last-word-position "a" "ab a c")  ==>  3
  (string-last-word-position "a" "ab ac")   ==>  #f
)

(load-from-library "string-parse.scm")
(test
  (string-parse " ?" " to be  or  not to be? ")
    ==>  ("to" "be" "or" "not" "to" "be")
)

(load-from-library "string-position.scm")
(test
  (string-position "ein" "gemeinsam")     ==>  3
  (string-position "people" "democracy")  ==>  #f
  (string-word-position "me" "test me")   ==>  5
  (string-word-position "me" "testme")    ==>  #f
)

(load-from-library "string-scan.scm")
(test
  (string-scan #\y "xyz")  ==>  1
)

(load-from-library "string-split.scm")
(test
  (string-split #\: "a::b:c:")  ==>  ("a" "" "b" "c" "")
)

(load-from-library "string-translate.scm")
(test
  (string-translate "a:b:c" ":" "-")  ==> "a-b-c"
;
  (string-translate
    "hello, world!"
    "abcdefghijklmnopqrstuvwxyz"
    "nopqrstuvwxyzabcdefghijklm")     ==>  "uryyb, jbeyq!"
)

(load-from-library "string-unsplit.scm")
(test
  (string-unsplit #\: '("" "a" "b" "" "c"))  ==>  ":a:b::c"
)

(load-from-library "substitute.scm")
(test
  (substitute '(* (+ 5 7) 9) '(((+ 5 7) . 12)))  ==>  (* 12 9)
)

(load-from-library "sum.scm")
(test
  (sum 2 5)  ==>  14
)

(load-from-library "symbols.scm")
(test
  (s9fes-syntax-objects)  ==>  ()
)

(load-from-library "syntax-rules.scm")
(test
(apply (syntax-rules ()
         ((_ p c ...)
           (if p (begin c ...))))
       '((< 1 2)
         (display "true")
         (newline)
         #t))
                       ==> (if (< 1 2)
                               (begin (display "true")
                                      (newline)
                                      #t))
)

(load-from-library "transpose.scm")
(test
  (transpose '((1 2 3) (4 5 6)))  ==>  ((1 4) (2 5) (3 6))
)

(load-from-library "union.scm")
(test
  (union '(v w x) '(w x y) '(x y z))  ==>  (v w x y z)
)

(load-from-library "unsort.scm")
(test
  (unsort '(1 2 3 4 5) 1)  ==>  (1 3 5 4 2)
)

(load-from-library "url-decode.scm")
(test
  (url-decode "%46%4F%4FBAR")  ==>  "FOOBAR"
)

(load-from-library "vector-mapb.scm")
(test
  (let ((v (vector 1 2 3)))
    (vector-map! - v)
    v)                       ==>  #(-1 -2 -3)
)

(load-from-library "write-to-string.scm")
(test
  (write-to-string '(a 1 #\c #(v) #t "str" "\"s\"" (a . d)))
    ==>  "(a 1 #\\c #(v) #t \"str\" \"\\\"s\\\"\" (a . d))"
;
  (display-to-string '(a 1 #\c #(v) #t "str" "\"s\"" (a . d)))
    ==>  "(a 1 c #(v) #t str \"s\" (a . d))"
)

(load-from-library "format.scm")
(test
  (format #f "~A ~:* ~S" '(#\c "s"))
    ==>  "(c s)  (#\\c \"s\")"
;
  (format #f "~20,'_,',,3:D" 123456789)
    ==>  "_________123,456,789"
;
  (format #f "~@{ ~A,~A ~}" 'a 1 'b 2 'c 3)
    ==>  " a,1  b,2  c,3 "
)

(load-from-library "pretty-print.scm")
(test
  (pp-string '("(let ((a 1) (b 2))"
               "(cons a b))"))
                                   ==> ("(let ((a 1)"
                                        "      (b 2))"
                                        "  (cons a b))")
)

(load-from-library "prolog.scm")
(test
  (begin (! (man socrates))
         (:- (mortal ?x)
             (man ?x))
         (query '(mortal ?who)))  ==>  (((who . socrates)))
)

(load-from-library "string-locate.scm")
(test
  (string-locate "test" "This is a test string")     ==>  10
  (string-locate "TEST" "This is a test string")     ==>  #f
)

(load-from-library "basename.scm")
(test
  (basename "/foo/bar/baz")     ==>  "baz"
  (basename "/goo/bar.Z" ".Z")  ==>  "bar"
)

(load-from-library "dirname.scm")
(test
  (dirname "/foo/bar/baz")  ==>  "/foo/bar"
  (dirname "foo/bar")       ==>  "foo"
  (dirname "foo/bar/")      ==>  "foo"
  (dirname "/foo")          ==>  "/"
  (dirname "/")             ==>  "/"
  (dirname "foo")           ==>  "."
)

(load-from-library "format-time.scm")
(test
  (format-time "~w ~4y-~@m-~2d ~2h:~2m:~2s"
               '(1 2009 3 9 8 53 20))
      ==> "Tue 2009-Apr-09 08:53:20"
)

(load-from-library "leap-yearp.scm")
(test
  (leap-year? 2000)  ==>  #t
  (leap-year? 2003)  ==>  #f
)

(load-from-library "mode-to-string.scm")
(test
  (mode->string #o5751)  ==>  "rwsr-x--t"
)

(load-from-library "parse-optionsb.scm")
(test
  (option #\o 'string)             ==>  (#\o string (#f))
  (opt-char (option #\o 'string))  ==>  #\o
  (opt-arg? (option #\o 'string))  ==>  #t
  (opt-type (option #\o 'string))  ==>  string
  (opt-val  (option #\o 'string))  ==>  #f
;
  (parse-options! '("-o" "file1" "file2")
                  `(,(option #\o #t))
                  #f)                   ==>  ("file2")
)

(load-from-library "proper-timep.scm")
(test
  (proper-time? '(3 1970 1 1 0 0 0))  ==>  #t
)

(load-from-library "time-to-unix-time.scm")
(test
  (time->unix-time '(6 2010 4 25 7 1 19))  ==>  1272178879
)

(load-from-library "unix-time-to-time.scm")
(test
  (unix-time->time 1272178879)  ==>  (6 2010 4 25 7 1 19)
)

(if (= 0 Errors)
    (begin (display "Everything fine!")
           (newline)))
